@@ -1,4 +1,5 @@
 import logging
+import os
 from urllib.parse import urlparse
 from copy import copy
 
@@ -8,7 +9,7 @@ import dvc.prompt as prompt
 from dvc.cache import NamedCache
 from dvc.exceptions import CollectCacheError, RemoteCacheRequiredError
 from dvc.exceptions import DvcException
-from dvc.remote.base import RemoteBASE
+from dvc.remote.base import BaseRemote
 
 
 logger = logging.getLogger(__name__)
@@ -28,7 +29,7 @@ class OutputIsNotFileOrDirError(DvcException):
 
 class OutputAlreadyTrackedError(DvcException):
     def __init__(self, path):
-        msg = "output '{}' is already tracked by scm (e.g. git)".format(path)
+        msg = "output '{}' is already tracked by SCM (e.g. Git)".format(path)
         super().__init__(msg)
 
 
@@ -37,10 +38,10 @@ class OutputIsStageFileError(DvcException):
         super().__init__("Stage file '{}' cannot be an output.".format(path))
 
 
-class OutputBase(object):
+class BaseOutput(object):
     IS_DEPENDENCY = False
 
-    REMOTE = RemoteBASE
+    REMOTE = BaseRemote
 
     PARAM_PATH = "path"
     PARAM_CACHE = "cache"
@@ -58,8 +59,6 @@ class OutputBase(object):
         },
     )
 
-    PARAM_TAGS = "tags"
-
     DoesNotExistError = OutputDoesNotExistError
     IsNotFileOrDirError = OutputIsNotFileOrDirError
     IsStageFileError = OutputIsStageFileError
@@ -75,7 +74,6 @@ class OutputBase(object):
         cache=True,
         metric=False,
         persist=False,
-        tags=None,
     ):
         self._validate_output_path(path)
         # This output (and dependency) objects have too many paths/urls
@@ -96,7 +94,6 @@ class OutputBase(object):
         self.use_cache = False if self.IS_DEPENDENCY else cache
         self.metric = False if self.IS_DEPENDENCY else metric
         self.persist = persist
-        self.tags = None if self.IS_DEPENDENCY else (tags or {})
 
         self.path_info = self._parse_path(remote, path)
         if self.use_cache and self.cache is None:
@@ -147,10 +144,21 @@ class OutputBase(object):
     def cache_path(self):
         return self.cache.checksum_to_path_info(self.checksum).url
 
+    @property
+    def checksum_type(self):
+        return self.remote.PARAM_CHECKSUM
+
     @property
     def checksum(self):
         return self.info.get(self.remote.PARAM_CHECKSUM)
 
+    @checksum.setter
+    def checksum(self, checksum):
+        self.info[self.remote.PARAM_CHECKSUM] = checksum
+
+    def get_checksum(self):
+        return self.remote.get_checksum(self.path_info)
+
     @property
     def is_dir_checksum(self):
         return self.remote.is_dir_checksum(self.checksum)
@@ -159,8 +167,11 @@ class OutputBase(object):
     def exists(self):
         return self.remote.exists(self.path_info)
 
+    def save_info(self):
+        return self.remote.save_info(self.path_info)
+
     def changed_checksum(self):
-        return self.checksum != self.remote.get_checksum(self.path_info)
+        return self.checksum != self.get_checksum()
 
     def changed_cache(self, filter_info=None):
         if not self.use_cache or not self.checksum:
@@ -215,7 +226,7 @@ class OutputBase(object):
             self.repo.scm.ignore(self.fspath)
 
         if not self.use_cache:
-            self.info = self.remote.save_info(self.path_info)
+            self.info = self.save_info()
             if self.metric:
                 self.verify_metric()
             if not self.IS_DEPENDENCY:
@@ -234,7 +245,7 @@ class OutputBase(object):
             )
             return
 
-        self.info = self.remote.save_info(self.path_info)
+        self.info = self.save_info()
 
     def commit(self):
         if self.use_cache:
@@ -259,9 +270,6 @@ class OutputBase(object):
         ret[self.PARAM_METRIC] = self.metric
         ret[self.PARAM_PERSIST] = self.persist
 
-        if self.tags:
-            ret[self.PARAM_TAGS] = self.tags
-
         return ret
 
     def verify_metric(self):
@@ -276,7 +284,6 @@ class OutputBase(object):
         self,
         force=False,
         progress_callback=None,
-        tag=None,
         relink=False,
         filter_info=None,
     ):
@@ -287,14 +294,9 @@ class OutputBase(object):
                 )
             return None
 
-        if tag:
-            info = self.tags[tag]
-        else:
-            info = self.info
-
         return self.cache.checkout(
             self.path_info,
-            info,
+            self.info,
             force=force,
             progress_callback=progress_callback,
             relink=relink,
@@ -383,11 +385,21 @@ class OutputBase(object):
             else:
                 return cache
 
+        path = str(self.path_info)
+        filter_path = str(filter_info) if filter_info else None
+        is_win = os.name == "nt"
         for entry in self.dir_cache:
             checksum = entry[self.remote.PARAM_CHECKSUM]
-            info = self.path_info / entry[self.remote.PARAM_RELPATH]
-            if not filter_info or info.isin_or_eq(filter_info):
-                cache.add(self.scheme, checksum, str(info))
+            entry_relpath = entry[self.remote.PARAM_RELPATH]
+            if is_win:
+                entry_relpath = entry_relpath.replace("/", os.sep)
+            entry_path = os.path.join(path, entry_relpath)
+            if (
+                not filter_path
+                or entry_path == filter_path
+                or entry_path.startswith(filter_path + os.sep)
+            ):
+                cache.add(self.scheme, checksum, entry_path)
 
         return cache
 
@@ -420,9 +432,9 @@ class OutputBase(object):
             if self.exists:
                 msg += (
                     "\n"
-                    "You can also use `dvc commit {stage}` to associate "
-                    "existing '{out}' with '{stage}'.".format(
-                        out=self, stage=self.stage.relpath
+                    "You can also use `dvc commit {stage.addressing}` "
+                    "to associate existing '{out}' with {stage}.".format(
+                        out=self, stage=self.stage
                     )
                 )
             logger.warning(msg)
@@ -433,13 +445,15 @@ class OutputBase(object):
         if not self.is_dir_checksum:
             return ret
 
-        ret.update(self._collect_used_dir_cache(**kwargs))
+        ret.add_child_cache(
+            self.checksum, self._collect_used_dir_cache(**kwargs),
+        )
 
         return ret
 
     @classmethod
     def _validate_output_path(cls, path):
-        from dvc.stage import Stage
+        from dvc.dvcfile import is_valid_filename
 
-        if Stage.is_valid_filename(path):
+        if is_valid_filename(path):
             raise cls.IsStageFileError(path)
