@@ -1,3 +1,4 @@
+import logging
 import os
 from contextlib import contextmanager
 from functools import wraps
@@ -18,6 +19,7 @@ from dvc.path_info import PathInfo
 from dvc.remote.base import RemoteActionNotImplemented
 from dvc.utils.fs import path_isin
 from .graph import check_acyclic, get_pipeline, get_pipelines
+from ..utils import parse_target
 
 
 def locked(f):
@@ -67,8 +69,8 @@ class Repo(object):
         from dvc.cache import Cache
         from dvc.data_cloud import DataCloud
         from dvc.repo.metrics import Metrics
+        from dvc.repo.params import Params
         from dvc.scm.tree import WorkingTree
-        from dvc.repo.tag import Tag
         from dvc.utils.fs import makedirs
 
         root_dir = self.find_root(root_dir)
@@ -84,7 +86,8 @@ class Repo(object):
         self.tree = WorkingTree(self.root_dir)
 
         self.tmp_dir = os.path.join(self.dvc_dir, "tmp")
-        makedirs(self.tmp_dir, exist_ok=True)
+        self.index_dir = os.path.join(self.tmp_dir, "index")
+        makedirs(self.index_dir, exist_ok=True)
 
         hardlink_lock = self.config["core"].get("hardlink_lock", False)
         self.lock = make_lock(
@@ -102,7 +105,7 @@ class Repo(object):
         self.cloud = DataCloud(self)
 
         self.metrics = Metrics(self)
-        self.tag = Tag(self)
+        self.params = Params(self)
 
         self._ignore()
 
@@ -189,41 +192,51 @@ class Repo(object):
         if not getattr(self, "_skip_graph_checks", False):
             self._collect_graph(self.stages + new_stages)
 
+    def _collect_inside(self, path, graph):
+        import networkx as nx
+
+        stages = nx.dfs_postorder_nodes(graph)
+        return [stage for stage in stages if path_isin(stage.path, path)]
+
     def collect(self, target, with_deps=False, recursive=False, graph=None):
         import networkx as nx
-        from dvc.stage import Stage
+        from ..dvcfile import Dvcfile
 
         if not target:
             return list(graph) if graph else self.stages
 
-        target = os.path.abspath(target)
-
         if recursive and os.path.isdir(target):
-            stages = nx.dfs_postorder_nodes(graph or self.graph)
-            return [stage for stage in stages if path_isin(stage.path, target)]
+            return self._collect_inside(
+                os.path.abspath(target), graph or self.graph
+            )
 
-        stage = Stage.load(self, target)
-
-        # Optimization: do not collect the graph for a specific target
+        path, name = parse_target(target)
+        dvcfile = Dvcfile(self, path)
+        stages = list(dvcfile.stages.filter(name).values())
         if not with_deps:
-            return [stage]
+            return stages
 
-        pipeline = get_pipeline(get_pipelines(graph or self.graph), stage)
-        return list(nx.dfs_postorder_nodes(pipeline, stage))
+        res = set()
+        for stage in stages:
+            pipeline = get_pipeline(get_pipelines(graph or self.graph), stage)
+            res.update(nx.dfs_postorder_nodes(pipeline, stage))
+        return res
 
     def collect_granular(self, target, *args, **kwargs):
-        from dvc.stage import Stage
+        from ..dvcfile import Dvcfile, is_valid_filename
 
         if not target:
             return [(stage, None) for stage in self.stages]
 
-        # Optimization: do not collect the graph for a specific .dvc target
-        if Stage.is_valid_filename(target) and not kwargs.get("with_deps"):
-            return [(Stage.load(self, target), None)]
+        file, name = parse_target(target)
+        if is_valid_filename(file) and not kwargs.get("with_deps"):
+            # Optimization: do not collect the graph for a specific .dvc target
+            stages = Dvcfile(self, file).stages.filter(name)
+            return [(stage, None) for stage in stages.values()]
 
         try:
-            (out,) = self.find_outs_by_path(target, strict=False)
-            filter_info = PathInfo(os.path.abspath(target))
+            (out,) = self.find_outs_by_path(file, strict=False)
+            filter_info = PathInfo(os.path.abspath(file))
             return [(out.stage, filter_info)]
         except OutputNotFoundError:
             stages = self.collect(target, *args, **kwargs)
@@ -251,8 +264,9 @@ class Repo(object):
         `all_branches`/`all_tags`/`all_commits` to expand the scope.
 
         Returns:
-            A dictionary with Schemes (representing output's location) as keys,
-            and a list with the outputs' `dumpd` as values.
+            A dictionary with Schemes (representing output's location) mapped
+            to items containing the output's `dumpd` names and the output's
+            children (if the given output is a directory).
         """
         from dvc.cache import NamedCache
 
@@ -284,7 +298,7 @@ class Repo(object):
 
         return cache
 
-    def _collect_graph(self, stages=None):
+    def _collect_graph(self, stages):
         """Generate a graph by using the given stages on the given directory
 
         The nodes of the graph are the stage's path relative to the root.
@@ -333,10 +347,9 @@ class Repo(object):
 
         G = nx.DiGraph()
         stages = stages or self.stages
-        stages = [stage for stage in stages if stage]
         outs = Trie()  # Use trie to efficiently find overlapping outs and deps
 
-        for stage in stages:
+        for stage in filter(bool, stages):
             for out in stage.outs:
                 out_key = out.path_info.parts
 
@@ -359,9 +372,9 @@ class Repo(object):
                         "rerun command with non overlapping outs paths."
                     ).format(
                         str(parent),
-                        parent.stage.relpath,
+                        parent.stage.addressing,
                         str(overlapping),
-                        overlapping.stage.relpath,
+                        overlapping.stage.addressing,
                     )
                     raise OverlappingOutputPathsError(parent, overlapping, msg)
 
@@ -385,14 +398,13 @@ class Repo(object):
                     overlapping.extend(outs.values(prefix=dep_key))
 
                 G.add_edges_from((stage, out.stage) for out in overlapping)
-
         check_acyclic(G)
 
         return G
 
     @cached_property
     def graph(self):
-        return self._collect_graph()
+        return self._collect_graph(self.stages)
 
     @cached_property
     def pipelines(self):
@@ -409,25 +421,28 @@ class Repo(object):
         NOTE: For large repos, this could be an expensive
               operation. Consider using some memoization.
         """
-        from dvc.stage import Stage
+        return self._collect_stages()
+
+    def _collect_stages(self):
+        from dvc.dvcfile import Dvcfile, is_valid_filename
 
         stages = []
         outs = set()
 
         for root, dirs, files in self.tree.walk(self.root_dir):
-            for fname in files:
-                path = os.path.join(root, fname)
-                if not Stage.is_valid_filename(path):
-                    continue
-                stage = Stage.load(self, path)
-                stages.append(stage)
-
-                for out in stage.outs:
-                    if out.scheme == "local":
-                        outs.add(out.fspath)
-
+            for file_name in filter(is_valid_filename, files):
+                path = os.path.join(root, file_name)
+                stage_loader = Dvcfile(self, path).stages
+                with stage_loader.log_level(at=logging.DEBUG):
+                    stages.extend(stage_loader.values())
+                outs.update(
+                    out.fspath
+                    for stage in stages
+                    for out in (
+                        out for out in stage.outs if out.scheme == "local"
+                    )
+                )
             dirs[:] = [d for d in dirs if os.path.join(root, d) not in outs]
-
         return stages
 
     def find_outs_by_path(self, path, outs=None, recursive=False, strict=True):
