@@ -1,4 +1,4 @@
-import dvc.utils.diff
+from dvc.utils.diff import format_dict, diff as _diff
 from .show import NoParamsError
 
 
@@ -10,21 +10,12 @@ def _get_params(repo, *args, rev=None, **kwargs):
         return {}
 
 
-def _format(params):
-    ret = {}
-    for key, val in params.items():
-        if isinstance(val, dict):
-            new_val = _format(val)
-        elif isinstance(val, list):
-            new_val = str(val)
-        else:
-            new_val = val
-        ret[key] = new_val
-    return ret
-
-
 def diff(repo, *args, a_rev=None, b_rev=None, **kwargs):
+    with_unchanged = kwargs.pop("all", False)
+
     old = _get_params(repo, *args, **kwargs, rev=(a_rev or "HEAD"))
     new = _get_params(repo, *args, **kwargs, rev=b_rev)
 
-    return dvc.utils.diff.diff(_format(old), _format(new))
+    return _diff(
+        format_dict(old), format_dict(new), with_unchanged=with_unchanged
+    )
